const path = require('path');
// const HtmlWebpackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const { VueLoaderPlugin } = require('vue-loader');
const webpack = require('webpack');
const ProgressBarPlugin = require('progress-bar-webpack-plugin');
const chalk = require('chalk');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');

module.exports = {
  entry: {
    'vue-loading-spinner': path.resolve(__dirname, '../src/index.js')
  },
  // devtool: '#source-map',
  mode: 'production',
  // 使用开发服务器, 将服务运行在dist目录中(其实是虚拟于内存中的)
  // 为了解决第三方包的路径问题, 我们将'./dist'改为'./'
  // 解决index.html输出到dist的问题
  plugins: [
    // new HtmlWebpackPlugin(),
    new CleanWebpackPlugin(),
    new MiniCssExtractPlugin({
      filename: '[name].css',
      chunkFilename: "[id].css"
    }),
    new VueLoaderPlugin(),
    new webpack.NamedModulesPlugin(),
    new webpack.HotModuleReplacementPlugin(),
    new ProgressBarPlugin(
      {
        format: chalk.blueBright(' build :bar :percent (:elapsed seconds) '),
        clear: false,
        summary: false,
        customSummary: res => {
          process.stderr.write(chalk.blueBright.bold(` build end use time ${res} \n`))
        }
      }
    ),
  ],
  // 单独 加载使用第三方包
  externals: {
    vue: 'Vue'
  },
  // 设置 js 输出位置
  output: {
    path: path.resolve(__dirname, '../dist'),
    filename: '[name].js',
    library: 'VueLoadingSpinner',
  },
  module: {
    rules: [
      // 解决加载css资源
      {
        test: /\.css$/,
        use: [
          'style-loader',
          'css-loader'
        ]
      },
      // 解决加载图片资源
      {
        test: /\.(png|svg|jpg|gif)$/,
        use: [
          'file-loader'
        ]
      },
      // 解决加载 less资源
      {
        test: /\.less$/,
        use: [
          {
            loader: MiniCssExtractPlugin.loader
          },
          {
            loader: "css-loader"
          }, {
            loader: "postcss-loader",
            options: { plugins: [require("autoprefixer")] }
          }, {
            loader: "less-loader",
          }
        ]
      },
      // 解决es6转为es5
      {
        test: /\.js$/,
        use: {
          loader: 'babel-loader',
          options: {
            plugins: ['@babel/plugin-proposal-class-properties', 'syntax-dynamic-import']
          }
        },
        // exclude: /(node_modules|bower_components)/,
        include: path.resolve(__dirname, '../src'),
      },
      // 支持vue的加载
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        include: path.resolve(__dirname, '../src'),
        options: {
          transformAssetUrls: {
            video: ['src', 'poster'],
            source: 'src',
            img: 'src',
            image: 'xlink:href'
          },
          compilerOptions: {
            preserveWhitespace: false
          }
        }
      },
    ]
  }
};